﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

//BRAIN DUMP: TODO:
//Save button action modify NoteModel object with Contents.
//On note select show dialog asking for save confirmation.
//About dialog.
//Note name check.


namespace Assignment2
{
    public class MainPageData : INotifyPropertyChanged
    {
        private Boolean _editorIsReadOnly;
        public Boolean EditorIsReadOnly
        {
            get
            {
                return _editorIsReadOnly;
            }

            set
            {
                _editorIsReadOnly = value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("EditorIsReadOnly"));
            }
        }

        private bool _editorEnabled = true;
        public bool EditorEnabled
        {
            get { return _editorEnabled; }
            set
            {
                _editorEnabled = value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("EditorEnabled"));
            }
        }

        private bool _disableEditor = false;
        public bool DisableEditor
        {
            get { return _disableEditor; }
            set
            {
                _disableEditor = value;
                EditorEnabled = !value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("DisableEditor"));
            }
        }

        private bool _newNote = true;
        public bool NewNote
        {
            get { return _newNote; }
            set
            {
                _newNote = value;
                if (_newNote)
                {
                    Title = "Untitled Note";
                    Contents = "";
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Title"));
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Contents"));
                    DisableEditor = false;
                }
            }
        }

        public string Title { get; set; } = "Untitled Note";
        public string Contents { get; set; } = "";

        public ObservableCollection<NoteModel> Notes { set; get; }
        private List<NoteModel> _allNotes = new List<NoteModel>();

        public event PropertyChangedEventHandler PropertyChanged;

        private NoteModel _selectedNote;
        public NoteModel SelectedNote
        {
            get
            {
                //Debug.WriteLine("Selected Note GET fired");
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("SelectedNote"));
                return _selectedNote;
            }
            set
            {
                //Debug.WriteLine("Selected Note SET fired");
                _selectedNote = value;
                if (value == null)
                {
                    NewNote = true;
                }
                else
                {
                    Title = value.Title;
                    Contents = value.Contents;
                    DisableEditor = true;
                    NewNote = false;
                }
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Title"));
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Contents"));

                SaveCommand.FireCanExecuteChanged();
                EditCommand.FireCanExecuteChanged();
                DeleteCommand.FireCanExecuteChanged();
            }
        }

        public AddCommand AddCommand { get; }

        public EditCommand EditCommand { get; }

        public SaveCommand SaveCommand { get; }

        public DeleteCommand DeleteCommand { get; }

        private string _filter;

        public string Filter
        {
            get
            {
                return _filter;
            }
            set
            {
                if (value == _filter)
                {
                    return;
                }
                _filter = value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Filter"));
                FilterList();
            }
        }

        private void FilterList()
        {
            Debug.WriteLine("Filter is firing....");
            if (_filter == null)
            {
                _filter = "";
            }
            var lowerCaseFilter = Filter.ToLowerInvariant();

            var result = _allNotes.Where(
                d => d.Title.ToLowerInvariant().Contains(lowerCaseFilter)).ToList();

            var toRemove = Notes.Except(result).ToList();

            foreach (var n in toRemove)
            {
                Notes.Remove(n);
            }

            var ResultCount = result.Count;
            for (int i = 0; i < ResultCount; i++)
            {
                var resultItem = result[i];
                if (i+1 > Notes.Count || !Notes[i].Equals(resultItem))
                {
                    Notes.Insert(i, resultItem);
                }
            }
        }

        public MainPageData()
        {
            AddCommand = new AddCommand(this);
            EditCommand = new EditCommand(this);
            SaveCommand = new SaveCommand(this);
            DeleteCommand = new DeleteCommand(this);

            Notes = new ObservableCollection<NoteModel>();

            SelectedNote = null;
            NewNote = true;
            DisableEditor = false;
            EditorIsReadOnly = false;

            //Repository.CreateDemoNote(this);

            if (Windows.ApplicationModel.DesignMode.DesignModeEnabled)
            {
                for (int i = 0; i < 5; i++)
                {
                    _allNotes.Add(new NoteModel("Note - " + i, "This is note number: " + i));
                }
                FilterList();
            }
            else
            {
                LoadData();
            }

        }

        //Consider new constructor for testing purposes only.

        private void LoadData()
        {
            _allNotes = Repository.GetAllNotes();
            FilterList();
        }

        public bool NoteWithTitleExists(String Title)
        {
            foreach (NoteModel Note in Notes)
            {
                if (Note.Title == Title)
                {
                    return true;
                }
            }
            return false;
        }
       
        public void DeleteNote(NoteModel Note)
        {
            _allNotes.Remove(Note);
            Repository.DeleteExistingNote(Note);
            FilterList();
        }

        public void SaveNewNote(NoteModel Note)
        {
            _allNotes.Add(Note);
            Repository.SaveNewNote(Note);
            FilterList();
        }
            
        public void SaveExistingNote(NoteModel Note)
        {
            Repository.SaveExistingNote(Note);
        }

        public void AddNoteToList(NoteModel Note)
        {
            _allNotes.Add(Note);
            FilterList();
        }
    }
}
