﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Assignment2;
using Windows.System.Threading;
using System.Collections.Generic;
using Microsoft.EntityFrameworkCore;
using System.Diagnostics;

namespace UnitTestProject
{
    [TestClass]
    public class UnitTest1
    {
        UnitTest1()
        {
            using (var db = new NoteContext())
            {
                db.Database.Migrate();
            }
        }

        [TestMethod]
        public void NoteModelCreateNote()
        {
            String Title = "TITLE kXMGjA4B9o";
            String Content = "CONTENT ZNHyrni3RW";
            NoteModel NM = new NoteModel(Title, Content);

            Assert.AreEqual(NM.Title, Title);
            Assert.AreEqual(NM.Contents, Content);
        }

        [TestMethod]
        public void NoteModelEditNote()
        {
            String Title = "TITLE kXMGjA4B9o";
            String Content = "CONTENT ZNHyrni3RW";
            NoteModel NM = new NoteModel(Title, Content);

            String NewTitle = "TITLE NEW: 6jvfKuqjm8";
            String NewContent = "CONTENT NEW: HvZ9SUpp7Y";

            NM.Title = NewTitle;
            NM.Contents = NewContent;

            Assert.AreEqual(NM.Title, NewTitle);
            Assert.AreEqual(NM.Contents, NewContent);
        }

        [TestMethod]
        public void NoteWithTitleAlreadyExists()
        {
            String Title = "TITLE: TQChWbBtna";
            String Content = "CONTENT: KKJvbLj0Sv";

            String NewTitle = "TITLE: WTXQSEIzGR";

            MainPageData mpd = new MainPageData();
            mpd.Notes.Add(new NoteModel(Title, Content));

            Assert.IsTrue(mpd.NoteWithTitleExists(Title));
            Assert.IsFalse(mpd.NoteWithTitleExists(NewTitle));
        }

        [TestMethod]
        public void SaveCommandCanFireWhenEditorNotDisabled()
        {

            MainPageData mpd = new MainPageData();
            SaveCommand sc = new SaveCommand(mpd);

            mpd.DisableEditor = false;

            Assert.IsTrue(sc.CanExecute(mpd));
        }

        [TestMethod]
        public void SaveCommandCanNotFireWhenEditorDisabled()
        {
            MainPageData mpd = new MainPageData();
            SaveCommand sc = new SaveCommand(mpd);

            mpd.DisableEditor = true;

            Assert.IsFalse(sc.CanExecute(mpd));
        }

        [TestMethod]
        public void EditCommandCanNotFireWhenEditorDisabledAndCurrentNoteIsNull()
        {
            MainPageData mpd = new MainPageData();
            EditCommand ec = new EditCommand(mpd);

            mpd.DisableEditor = true;

            Assert.IsFalse(ec.CanExecute(mpd));
        }

        [TestMethod]
        public void EditCommandCanFireWhenEditorDisabledAndCurrentNoteIsNotNull()
        {
            String Title = "TITLE: X7N7rg2lE5";
            String Content = "CONTENT: 24YOo31lrD";

            MainPageData mpd = new MainPageData();
            EditCommand ec = new EditCommand(mpd);
            mpd.SelectedNote = new NoteModel(Title, Content);

            mpd.DisableEditor = true;

            Assert.IsTrue(ec.CanExecute(mpd));
        }

        [TestMethod]
        public void EditCommandNotCanFireWhenEditorEnabledAndCurrentNoteIsNotNull()
        {
            String Title = "TITLE: X7N7rg2lE5";
            String Content = "CONTENT: 24YOo31lrD";

            MainPageData mpd = new MainPageData();
            EditCommand ec = new EditCommand(mpd);
            mpd.SelectedNote = new NoteModel(Title, Content);

            mpd.DisableEditor = false;

            Assert.IsFalse(ec.CanExecute(mpd));
        }

        [TestMethod]
        public void DeleteCommandCanFireWhenSelectedNoteIsNotNull()
        {
            String Title = "TITLE: got4ofjBnZ";
            String Content = "CONTENT: DkmVPy6T2s";

            MainPageData mpd = new MainPageData();
            DeleteCommand dc = new DeleteCommand(mpd);
            mpd.SelectedNote = new NoteModel(Title, Content);

            Assert.IsTrue(dc.CanExecute(mpd));
        }

        [TestMethod]
        public void DeleteCommandCanFireWhenSelectedNoteIsNullAndNoteIsNew()
        {
            MainPageData mpd = new MainPageData();
            DeleteCommand dc = new DeleteCommand(mpd);
            mpd.NewNote = true;

            Assert.IsTrue(dc.CanExecute(mpd));
        }

        [TestMethod]
        public void DeleteCommandCanNotFireWhenSelectedNoteIsNullAndNoteIsNotNew()
        {
            MainPageData mpd = new MainPageData();
            DeleteCommand dc = new DeleteCommand(mpd);
            
            //Order is very important here as setting the selected note to null enables a new note.
            mpd.SelectedNote = null;
            mpd.NewNote = false;

            Assert.IsFalse(dc.CanExecute(mpd));
        }

        [TestMethod]
        public void FilterShowsOnlySpecifiedTarget()
        {
            String Title = "TITLE- xqZaH3BzHJ";
            String Content = "CONTENT- ukJhsLOLBW";

            MainPageData mpd = new MainPageData();
            mpd.Notes.Add(new NoteModel(Title, Content));

            mpd.Filter = "Title";

            TimeSpan seconds = TimeSpan.FromSeconds(2);
            ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
                {
                    Assert.AreEqual(1, mpd.Notes.Count);
                    Assert.AreEqual(mpd.Notes[0].Title, Title);
                }, seconds);
        }

        [TestMethod]
        public void FilterShowsAllIfFilterEmpty()
        {
            String Title = "TITLE- ZcNapjxLEr";
            String Content = "CONTENT- wCsf9S9kmB";

            MainPageData mpd = new MainPageData();
            mpd.Notes.Add(new NoteModel(Title, Content));

            int PreFilterListCount = mpd.Notes.Count;

            mpd.Filter = "";

            TimeSpan seconds = TimeSpan.FromSeconds(2);
            ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                Assert.AreEqual(PreFilterListCount, mpd.Notes.Count);
            }, seconds);
        }

        //[TestMethod]
        //public void SaveNewNoteToDisk()
        //{
        //    String Title = "TITLE- zt5HOpiF7n";
        //    String Content = "CONTENT-46Ywebo6Oc";

        //    NoteModel note = new NoteModel(Title, Content);

        //    Repository.SaveNewNote(note);

        //    TimeSpan seconds = TimeSpan.FromSeconds(2);
        //    ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
        //    {
        //        //List<NoteModel> = Repository.GetAllNotesAsync();



        //    }, seconds);

        //}

        //NEW TEST METHODS TO TEST DB

        [TestMethod]
        public void SetUpDB()
        {

        }

        [TestMethod]
        public void SaveNewNoteToDbAndRetrieve()
        {
            NoteModel testNote;
            string title = "This is Only A Test SjHBKfsbI0";
            string body = "This is a sample test note. It is for testing purposes only. sWVNEyl6Sv";

            testNote = new NoteModel(title, body);
            Repository.SaveNewNote(testNote);

            List<NoteModel> notes = new List<NoteModel>();
            TimeSpan seconds = TimeSpan.FromSeconds(2);
            ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                notes = Repository.GetAllNotes();

                bool passed = false;

                foreach (NoteModel note in notes)
                {
                    Debug.WriteLine(note.Title);
                    if (note.Title == title && note.Contents == body)
                    {
                        passed = true;
                        break;
                    }
                }

                Assert.IsTrue(passed);
            }, seconds);

        }

        [TestMethod]
        public void EditExistingNoteAndRetrieve()
        {
            NoteModel testNote = null;
            string OriginalTitle = "This is Only A Test SjHBKfsbI0";
            string OriginalBody = "This is a sample test note. It is for testing purposes only. sWVNEyl6Sv";

            List<NoteModel> notes = new List<NoteModel>();

            TimeSpan seconds = TimeSpan.FromSeconds(2);
            ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                //Get list of notes and traverse the list looking for the testnote. Fail if not found.
                notes = Repository.GetAllNotes();

                foreach (NoteModel note in notes)
                {
                    if (note.Title == OriginalTitle && note.Contents == OriginalBody)
                    {
                        testNote = note;
                    }
                }

                if (testNote == null)
                {
                    Assert.Fail("Can not find Note in database.");
                }
            }, seconds);

            //New string values for note.
            string NewTitle = "This is Only A NEW Test Xz1YZ4q94w";
            string NewBody = "This is a NEW sample test note. It is for testing purposes only. Q8Gcu6jwk8";

            //Save Updated note
            TimeSpan seconds0 = TimeSpan.FromSeconds(4);
            ThreadPoolTimer timer0 = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {

                testNote.Title = NewTitle;
                testNote.Contents = NewBody;

                Repository.SaveExistingNote(testNote);
            }, seconds);

            //Look for updated note.
            TimeSpan seconds1 = TimeSpan.FromSeconds(6);
            ThreadPoolTimer timer1 = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                bool passed = false;

                foreach (NoteModel note in notes)
                {
                    if (note.Title == NewTitle && note.Contents == NewBody)
                    {
                        passed = true;
                        break;
                    }
                }
                Assert.IsTrue(passed);
            }, seconds);
        }

        [TestMethod]
        public void DeleteExistingNoteAndVerify()
        {
            NoteModel testNote = null;
            string NewTitle = "This is Only A NEW Test Xz1YZ4q94w";
            string NewBody = "This is a NEW sample test note. It is for testing purposes only. Q8Gcu6jwk8";

            List<NoteModel> notes = new List<NoteModel>();

            //Look for updated note.
            TimeSpan seconds = TimeSpan.FromSeconds(2);
            ThreadPoolTimer timer = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                //Get list of notes and traverse the list looking for the testnote. Fail if not found.
                notes = Repository.GetAllNotes();

                foreach (NoteModel note in notes)
                {
                    if (note.Title == NewTitle && note.Contents == NewBody)
                    {
                        testNote = note;
                    }
                }

                if (testNote == null)
                {
                    Assert.Fail("Can not find Note in database.");
                }
                else
                {
                    Repository.DeleteExistingNote(testNote);
                }

            }, seconds);

            bool passed = true;

            //Look for Deleted Note.
            TimeSpan seconds1 = TimeSpan.FromSeconds(6);
            ThreadPoolTimer timer1 = ThreadPoolTimer.CreatePeriodicTimer((testFunction) =>
            {
                notes = Repository.GetAllNotes();

                foreach (NoteModel note in notes)
                {
                    if (note.Title == NewTitle && note.Contents == NewBody)
                    {
                        passed = false;
                        break;
                    }
                }
            }, seconds);

            Assert.IsTrue(passed);
        }

    }
}
